@@ -212,6 +212,16 @@ class NeutronSecurityGroupUpdate(task_manager.Task):
         return client.neutron_client.update_security_group(**self.args)
 
 
+class NeutronSecurityGroupRuleCreate(task_manager.Task):
+    def main(self, client):
+        return client.neutron_client.create_security_group_rule(**self.args)
+
+
+class NeutronSecurityGroupRuleDelete(task_manager.Task):
+    def main(self, client):
+        return client.neutron_client.delete_security_group_rule(**self.args)
+
+
 class NovaSecurityGroupList(task_manager.Task):
     def main(self, client):
         return client.nova_client.security_groups.list()
@@ -232,6 +242,16 @@ class NovaSecurityGroupUpdate(task_manager.Task):
         return client.nova_client.security_groups.update(**self.args)
 
 
+class NovaSecurityGroupRuleCreate(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.security_group_rules.create(**self.args)
+
+
+class NovaSecurityGroupRuleDelete(task_manager.Task):
+    def main(self, client):
+        return client.nova_client.security_group_rules.delete(**self.args)
+
+
 # TODO: Do this with neutron instead of nova if possible
 class FloatingIPList(task_manager.Task):
     def main(self, client):
@@ -278,9 +298,19 @@ class ContainerUpdate(task_manager.Task):
         client.swift_client.post_container(**self.args)
 
 
+class ObjectCapabilities(task_manager.Task):
+    def main(self, client):
+        return client.swift_client.get_capabilities(**self.args)
+
+
+class ObjectDelete(task_manager.Task):
+    def main(self, client):
+        return client.swift_client.delete_object(**self.args)
+
+
 class ObjectCreate(task_manager.Task):
     def main(self, client):
-        client.swift_client.put_object(**self.args)
+        return client.swift_service.upload(**self.args)
 
 
 class ObjectUpdate(task_manager.Task):
