@@ -14,26 +14,46 @@
 
 
 import bunch
+import logging
 import six
 
+from neutronclient.common.exceptions import NeutronClientException
+
 from shade import exc
+from shade import _utils
+
 
 NON_CALLABLES = (six.string_types, bool, dict, int, list, type(None))
 
+log = logging.getLogger(__name__)
+
 
 def find_nova_addresses(addresses, ext_tag=None, key_name=None, version=4):
 
     ret = []
     for (k, v) in iter(addresses.items()):
-        if key_name and k == key_name:
-            ret.extend([addrs['addr'] for addrs in v
-                        if addrs['version'] == version])
-        else:
-            for interface_spec in v:
-                if ('OS-EXT-IPS:type' in interface_spec
-                        and interface_spec['OS-EXT-IPS:type'] == ext_tag
-                        and interface_spec['version'] == version):
-                    ret.append(interface_spec['addr'])
+        if key_name is not None and k != key_name:
+            # key_name is specified and it doesn't match the current network.
+            # Continue with the next one
+            continue
+
+        for interface_spec in v:
+            if ext_tag is not None:
+                if 'OS-EXT-IPS:type' not in interface_spec:
+                    # ext_tag is specified, but this interface has no tag
+                    # We could actually return right away as this means that
+                    # this cloud doesn't support OS-EXT-IPS. Nevertheless,
+                    # it would be better to perform an explicit check. e.g.:
+                    #   cloud._has_nova_extension('OS-EXT-IPS')
+                    # But this needs cloud to be passed to this function.
+                    continue
+                elif interface_spec['OS-EXT-IPS:type'] != ext_tag:
+                    # Type doesn't match, continue with next one
+                    continue
+
+            if interface_spec['version'] == version:
+                ret.append(interface_spec['addr'])
+
     return ret
 
 
@@ -52,6 +72,70 @@ def get_server_public_ip(server):
     return get_server_ip(server, ext_tag='floating', key_name='public')
 
 
+def get_server_external_ipv4(cloud, server):
+    if cloud.has_service('network'):
+        try:
+            # Search a fixed IP attached to an external net. Unfortunately
+            # Neutron ports don't have a 'floating_ips' attribute
+            server_ports = cloud.search_ports(
+                filters={'device_id': server.id})
+            ext_nets = cloud.search_networks(filters={'router:external': True})
+        except NeutronClientException as e:
+            log.debug(
+                "Something went wrong talking to neutron API: "
+                "'{msg}'. Trying with Nova.".format(msg=str(e)))
+            # Fall-through, trying with Nova
+        else:
+            for net in ext_nets:
+                for port in server_ports:
+                    if net['id'] == port['network_id']:
+                        for ip in port['fixed_ips']:
+                            if _utils.is_ipv4(ip['ip_address']):
+                                return ip['ip_address']
+            # The server doesn't have an interface on an external network so it
+            # can either have a floating IP or have no way to be reached from
+            # outside the cloud.
+            # Fall-through, trying with Nova
+
+    # The cloud doesn't support Neutron or Neutron can't be contacted. The
+    # server might have fixed addresses that are reachable from outside the
+    # cloud (e.g. Rax) or have plain ol' floating IPs
+
+    # Try to get an address from a network named 'public'
+    ext_ip = get_server_ip(server, key_name='public')
+    if ext_ip is not None:
+        return ext_ip
+
+    # Try to find a globally routable IP address
+    for interfaces in server.addresses.values():
+        for interface in interfaces:
+            if _utils.is_ipv4(interface['addr']) and \
+                    _utils.is_globally_routable_ipv4(interface['addr']):
+                return interface['addr']
+
+    # Last, try to get a floating IP address
+    ext_ip = get_server_ip(server, ext_tag='floating')
+    if ext_ip is not None:
+        return ext_ip
+
+    return None
+
+
+def get_server_external_ipv6(server):
+    """ Get an IPv6 address reachable from outside the cloud.
+
+    This function assumes that if a server has an IPv6 address, that address
+    is reachable from outside the cloud.
+
+    :param server: the server from which we want to get an IPv6 address
+    :return: a string containing the IPv6 address or None
+    """
+    addresses = find_nova_addresses(addresses=server.addresses, version=6)
+    if addresses:
+        return addresses[0]
+    return None
+
+
 def get_groups_from_server(cloud, server, server_vars):
     groups = []
 
@@ -99,7 +183,7 @@ def get_hostvars_from_server(cloud, server, mounts=None):
     server_vars.pop('links', None)
 
     # Fist, add an IP address
-    server_vars['public_v4'] = get_server_public_ip(server)
+    server_vars['public_v4'] = get_server_external_ipv4(cloud, server)
     server_vars['private_v4'] = get_server_private_ip(server)
     if cloud.private:
         interface_ip = server_vars['private_v4']
