@@ -56,10 +56,11 @@ class FakeProject(object):
 
 
 class FakeServer(object):
-    def __init__(self, id, name, status):
+    def __init__(self, id, name, status, addresses=None):
         self.id = id
         self.name = name
         self.status = status
+        self.addresses = addresses
 
 
 class FakeService(object):
@@ -97,3 +98,15 @@ class FakeSecgroup(object):
         self.name = name
         self.description = description
         self.rules = rules
+
+
+class FakeNovaSecgroupRule(object):
+    def __init__(self, id, from_port=None, to_port=None, ip_protocol=None,
+                 cidr=None, parent_group_id=None):
+        self.id = id
+        self.from_port = from_port
+        self.to_port = to_port
+        self.ip_protocol = ip_protocol
+        if cidr:
+            self.ip_range = {'cidr': cidr}
+        self.parent_group_id = parent_group_id
